@@ -12,8 +12,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import mock
 import testtools
+from unittest import mock
 import uuid
 
 import eventlet
